@@ -285,12 +285,12 @@ pdl_error pdl_dim_checks(
       pdl *pdl = pdls[i];
       PDL_Indx *dims = pdl->dims;
       for (j=0; j<ninds; j++) {
-        ind_id = PDL_IND_ID(vtable, i, j);
-        if (ind_sizes[ind_id] > 1 && ind_sizes[ind_id] != dims[j])
+        PDL_Indx ind_id = PDL_IND_ID(vtable, i, j), ind_sz = ind_sizes[ind_id];
+        if (ind_sz > 1 && ind_sz != dims[j])
           return pdl_make_error(PDL_EUSERERROR,
             "Error in %s: [phys] parameter '%s' index '%s' size %"IND_FLAG", but ndarray dim has size %"IND_FLAG"\n",
             vtable->name, vtable->par_names[i], vtable->ind_names[ind_id],
-            ind_sizes[ind_id], dims[j]
+            ind_sz, dims[j]
           );
       }
     }
@@ -422,10 +422,16 @@ pdl_error pdl_initbroadcaststruct(int nobl,
 	    if(pdls[j]->broadcastids[0]-         // If we're off the end of the current PDLs dimlist,
 	       realdims[j] <= nth)                    //    then just skip it.
 	      continue;
-	    if(pdls[j]->dims[nth+realdims[j]] != 1) { // If the current dim in the current PDL is not 1,
+	    PDL_Indx cur_pdl_dim = pdls[j]->dims[nth+realdims[j]];
+	    if (vtable && j >= vtable->nparents && cur_pdl_dim == 1 && cur_pdl_dim != broadcast->dims[nth])
+	      return pdl_make_error(PDL_EUSERERROR,
+		"Error in %s: output parameter '%s' implicit dim %"IND_FLAG" size %"IND_FLAG", but dim has size %"IND_FLAG"\n",
+		vtable->name, vtable->par_names[j], nth, broadcast->dims[nth],
+		cur_pdl_dim
+	      );
+	    if(cur_pdl_dim != 1) { // If the current dim in the current PDL is not 1,
 	      if(broadcast->dims[nth] != 1) {            //   ... and the current planned size isn't 1,
-		if(broadcast->dims[nth] !=
-		   pdls[j]->dims[nth+realdims[j]]) {  //   ... then check to make sure they're the same.
+		if(broadcast->dims[nth] != cur_pdl_dim) { //   ... then check to make sure they're the same.
 		  char buf0[BUFSIZ];
 		  buf0[0] = '\0';
 		  pdl_broadcast_mismatch_msg(
@@ -437,8 +443,7 @@ pdl_error pdl_initbroadcaststruct(int nobl,
 		/* If we're still here, they're the same -- OK! */
 
 	      } else {                                // current planned size is 1 -- mod it to match this PDL
-		broadcast->dims[nth] =
-		  pdls[j]->dims[nth+realdims[j]];
+		broadcast->dims[nth] = cur_pdl_dim;
 	      }
 
 	      PDL_BRC_INC(broadcast->incs, npdls, j, nth) =       // Update the corresponding data stride
